@@ -25,14 +25,15 @@ package org.wildfly.extension.messaging.activemq;
 import static org.jboss.as.controller.SimpleAttributeDefinitionBuilder.create;
 import static org.jboss.as.controller.client.helpers.MeasurementUnit.BYTES;
 import static org.jboss.as.controller.client.helpers.MeasurementUnit.MILLISECONDS;
+import static org.jboss.as.controller.registry.AttributeAccess.Flag.COUNTER_METRIC;
+import static org.jboss.as.controller.registry.AttributeAccess.Flag.GAUGE_METRIC;
 import static org.jboss.as.controller.registry.AttributeAccess.Flag.RESTART_ALL_SERVICES;
-import static org.jboss.dmr.ModelType.BIG_DECIMAL;
 import static org.jboss.dmr.ModelType.BOOLEAN;
+import static org.jboss.dmr.ModelType.DOUBLE;
 import static org.jboss.dmr.ModelType.INT;
 import static org.jboss.dmr.ModelType.LONG;
 import static org.wildfly.extension.messaging.activemq.jms.Validators.noDuplicateElements;
 
-import org.apache.activemq.artemis.api.config.ActiveMQDefaultConfiguration;
 import org.apache.activemq.artemis.api.core.client.ActiveMQClient;
 import org.apache.activemq.artemis.core.config.impl.FileConfiguration;
 import org.jboss.as.controller.AttributeDefinition;
@@ -58,67 +59,86 @@ public interface CommonAttributes {
     String ENTRIES = "entries";
     String MODULE = "module";
     String NAME = "name";
-
+    /**
+     * @see ActiveMQClient.DEFAULT_CALL_TIMEOUT
+     */
     AttributeDefinition CALL_TIMEOUT = create("call-timeout", LONG)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_CALL_TIMEOUT))
+            .setDefaultValue(new ModelNode(30000L))
             .setMeasurementUnit(MILLISECONDS)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
+            .setRestartAllServices()
             .build();
 
     SimpleAttributeDefinition CALL_FAILOVER_TIMEOUT = create("call-failover-timeout", LONG)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_CALL_FAILOVER_TIMEOUT))
-            .setAllowNull(true)
+            // ActiveMQClient.DEFAULT_CALL_FAILOVER_TIMEOUT was changed from -1 to 30000 in ARTEMIS-255
+            // we set it to 60s to leave more time for WildFly to failover
+            .setDefaultValue(new ModelNode(60000L))
+            .setRequired(false)
             .setAllowExpression(true)
             .setMeasurementUnit(MILLISECONDS)
+            .setRestartAllServices()
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_CLIENT_FAILURE_CHECK_PERIOD
+     */
     SimpleAttributeDefinition CHECK_PERIOD = create("check-period", LONG)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_CLIENT_FAILURE_CHECK_PERIOD))
-            .setAllowNull(true)
+            .setDefaultValue(new ModelNode(30000L))
+            .setRequired(false)
             .setAllowExpression(true)
             .setMeasurementUnit(MILLISECONDS)
+            .setValidator(InfiniteOrPositiveValidators.LONG_INSTANCE)
             .setFlags(RESTART_ALL_SERVICES)
             .build();
 
     SimpleAttributeDefinition CLIENT_ID = create("client-id", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
+            .setRestartAllServices()
             .build();
 
     AttributeDefinition CONSUMER_COUNT = create("consumer-count", INT)
             .setStorageRuntime()
-            .setAllowNull(true)
+            .setRequired(false)
+            .addFlag(GAUGE_METRIC)
             .build();
 
     SimpleAttributeDefinition BRIDGE_CONFIRMATION_WINDOW_SIZE = create("confirmation-window-size", INT)
             .setDefaultValue(new ModelNode(FileConfiguration.DEFAULT_CONFIRMATION_WINDOW_SIZE))
             .setMeasurementUnit(BYTES)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
+            .setCorrector(InfiniteOrPositiveValidators.NEGATIVE_VALUE_CORRECTOR)
+            .setValidator(InfiniteOrPositiveValidators.INT_INSTANCE)
             .setRestartAllServices()
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_CONNECTION_TTL
+     */
     SimpleAttributeDefinition CONNECTION_TTL = create("connection-ttl", LONG)
-            .setDefaultValue(new ModelNode().set(ActiveMQClient.DEFAULT_CONNECTION_TTL))
-            .setAllowNull(true)
+            .setDefaultValue(new ModelNode(60000L))
+            .setRequired(false)
             .setAllowExpression(true)
+            .setValidator(InfiniteOrPositiveValidators.LONG_INSTANCE)
             .setMeasurementUnit(MILLISECONDS)
             .setRestartAllServices()
             .build();
 
     SimpleAttributeDefinition DEAD_LETTER_ADDRESS = create("dead-letter-address", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .build();
 
     AttributeDefinition DELIVERING_COUNT = create("delivering-count", INT)
             .setStorageRuntime()
-            .setUndefinedMetricValue(new ModelNode(0))
+            .setUndefinedMetricValue(ModelNode.ZERO)
+            .addFlag(GAUGE_METRIC)
             .build();
 
     StringListAttributeDefinition DESTINATION_ENTRIES = new StringListAttributeDefinition.Builder(ENTRIES)
-            .setAllowNull(false)
+            .setRequired(true)
             .setListValidator(noDuplicateElements(new StringLengthValidator(1, false, true)))
             .setAllowExpression(true)
             .setAttributeParser(AttributeParser.STRING_LIST)
@@ -127,7 +147,7 @@ public interface CommonAttributes {
             .build();
 
     StringListAttributeDefinition LEGACY_ENTRIES = new StringListAttributeDefinition.Builder("legacy-entries")
-            .setAllowNull(true)
+            .setRequired(false)
             .setListValidator(noDuplicateElements(new StringLengthValidator(1, false, true)))
             .setAllowExpression(true)
             .setAttributeParser(AttributeParser.STRING_LIST)
@@ -136,8 +156,8 @@ public interface CommonAttributes {
             .build();
 
     SimpleAttributeDefinition DURABLE = create("durable", BOOLEAN)
-            .setDefaultValue(new ModelNode().set(true))
-            .setAllowNull(true)
+            .setDefaultValue(ModelNode.TRUE)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
@@ -148,20 +168,22 @@ public interface CommonAttributes {
             .build();
 
     SimpleAttributeDefinition EXPIRY_ADDRESS = create("expiry-address", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .build();
 
     SimpleAttributeDefinition FILTER = create("filter", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_HA
+     */
     SimpleAttributeDefinition HA = create("ha", BOOLEAN)
-            .setDefaultValue(new ModelNode()
-                    .set(ActiveMQClient.DEFAULT_HA))
-            .setAllowNull(true)
+            .setDefaultValue(ModelNode.FALSE)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
@@ -172,44 +194,61 @@ public interface CommonAttributes {
             .setXmlName("param")
             .build();
 
-    SimpleAttributeDefinition JGROUPS_STACK = create("jgroups-stack", ModelType.STRING)
-            .setAllowNull(true)
+    @Deprecated SimpleAttributeDefinition JGROUPS_CHANNEL_FACTORY = create("jgroups-stack", ModelType.STRING)
+            .setRequired(false)
             // do not allow expression as this may reference another resource
             .setAllowExpression(false)
-            .setRequires("jgroups-channel")
+            .setRequires("jgroups-cluster")
+            .setDeprecated(MessagingExtension.VERSION_3_0_0)
             .setRestartAllServices()
             .build();
 
     SimpleAttributeDefinition JGROUPS_CHANNEL = create("jgroups-channel", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
+            // do not allow expression as this may reference another resource
+            .setAllowExpression(false)
+            .setRequires("jgroups-cluster")
+            .setRestartAllServices()
+            .build();
+
+    SimpleAttributeDefinition JGROUPS_CLUSTER = create("jgroups-cluster", ModelType.STRING)
+            .setRequired(false)
             // do not allow expression as this may reference another resource
             .setAllowExpression(false)
             .setAlternatives("socket-binding")
             .setRestartAllServices()
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_MAX_RETRY_INTERVAL
+     */
     AttributeDefinition MAX_RETRY_INTERVAL = create("max-retry-interval", LONG)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_MAX_RETRY_INTERVAL))
+            .setDefaultValue(new ModelNode(2000L))
             .setMeasurementUnit(MILLISECONDS)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
 
     AttributeDefinition MESSAGE_COUNT = create("message-count", LONG)
             .setStorageRuntime()
-            .setUndefinedMetricValue(new ModelNode(0))
+            .setUndefinedMetricValue(ModelNode.ZERO)
+            .addFlag(GAUGE_METRIC)
             .build();
 
     AttributeDefinition MESSAGES_ADDED = create("messages-added", LONG)
             .setStorageRuntime()
-            .setUndefinedMetricValue(new ModelNode(0))
+            .setUndefinedMetricValue(ModelNode.ZERO)
+            .addFlag(COUNTER_METRIC)
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_MIN_LARGE_MESSAGE_SIZE
+     */
     AttributeDefinition MIN_LARGE_MESSAGE_SIZE = create("min-large-message-size", INT)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_MIN_LARGE_MESSAGE_SIZE))
+            .setDefaultValue(new ModelNode(100 * 1024))
             .setMeasurementUnit(BYTES)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
@@ -228,48 +267,55 @@ public interface CommonAttributes {
             .build();
 
     ObjectListAttributeDefinition INCOMING_INTERCEPTORS = ObjectListAttributeDefinition.Builder.of("incoming-interceptors", CommonAttributes.CLASS)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(false)
             .setMinSize(1)
             .setMaxSize(Integer.MAX_VALUE)
             .build();
 
     ObjectListAttributeDefinition OUTGOING_INTERCEPTORS = ObjectListAttributeDefinition.Builder.of("outgoing-interceptors", CommonAttributes.CLASS)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(false)
             .setMinSize(1)
             .setMaxSize(Integer.MAX_VALUE)
             .build();
 
+    /**
+     * @see ActiveMQClient.DEFAULT_RETRY_INTERVAL
+     */
     AttributeDefinition RETRY_INTERVAL = create("retry-interval", LONG)
-            .setDefaultValue(new ModelNode().set(ActiveMQClient.DEFAULT_RETRY_INTERVAL))
+            .setDefaultValue(new ModelNode(2000L))
             .setMeasurementUnit(MILLISECONDS)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
 
-    AttributeDefinition RETRY_INTERVAL_MULTIPLIER = create("retry-interval-multiplier", BIG_DECIMAL)
-            .setDefaultValue(new ModelNode(ActiveMQClient.DEFAULT_RETRY_INTERVAL_MULTIPLIER))
-            .setAllowNull(true)
+    /**
+     * @see ActiveMQClient.DEFAULT_RETRY_INTERVAL_MULTIPLIER
+     */
+    AttributeDefinition RETRY_INTERVAL_MULTIPLIER = create("retry-interval-multiplier", DOUBLE)
+            .setDefaultValue(new ModelNode(1.0d))
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
 
     AttributeDefinition SCHEDULED_COUNT = create("scheduled-count", LONG)
             .setStorageRuntime()
-            .setUndefinedMetricValue(new ModelNode(0))
+            .setUndefinedMetricValue(ModelNode.ZERO)
+            .addFlag(GAUGE_METRIC)
             .build();
 
     SimpleAttributeDefinition SELECTOR = create("selector", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(true)
             .setRestartAllServices()
             .build();
 
     SimpleAttributeDefinition SOCKET_BINDING = create("socket-binding", ModelType.STRING)
-            .setAllowNull(true)
-            .setAlternatives(JGROUPS_CHANNEL.getName())
+            .setRequired(false)
+            .setAlternatives(JGROUPS_CLUSTER.getName())
             .setRestartAllServices()
             .addAccessConstraint(SensitiveTargetAccessConstraintDefinition.SOCKET_BINDING_REF)
             .build();
@@ -279,15 +325,11 @@ public interface CommonAttributes {
             .build();
 
     SimpleAttributeDefinition TRANSFORMER_CLASS_NAME = create("transformer-class-name", ModelType.STRING)
-            .setAllowNull(true)
+            .setRequired(false)
             .setAllowExpression(false)
             .setRestartAllServices()
             .build();
 
-    SimpleAttributeDefinition USER = create("user", ModelType.STRING, true)
-            .setAllowExpression(true)
-            .setDefaultValue(new ModelNode(ActiveMQDefaultConfiguration.getDefaultClusterUser()))
-            .build();
 
     String ACCEPTOR = "acceptor";
     String ACCEPTORS = "acceptors";
@@ -303,6 +345,8 @@ public interface CommonAttributes {
     String BROADCAST_GROUPS = "broadcast-groups";
     String CHECK_FOR_LIVE_SERVER2 = "check-for-live-server";
     String CLASS_NAME = "class-name";
+    String EXTERNAL_JMS_QUEUE = "external-jms-queue";
+    String EXTERNAL_JMS_TOPIC = "external-jms-topic";
     String CLUSTER_CONNECTION = "cluster-connection";
     String CLUSTER_CONNECTIONS = "cluster-connections";
     String CLUSTER_NAME = "cluster-name";
@@ -345,6 +389,8 @@ public interface CommonAttributes {
     String IN_VM_CONNECTOR = "in-vm-connector";
     String LEGACY = "legacy";
     String LEGACY_CONNECTION_FACTORY = "legacy-connection-factory";
+    String JGROUPS_BROADCAST_GROUP = "jgroups-broadcast-group";
+    String JGROUPS_DISCOVERY_GROUP = "jgroups-discovery-group";
     String JMS_BRIDGE = "jms-bridge";
     String JMS_CONNECTION_FACTORIES = "jms-connection-factories";
     String JMS_DESTINATION = "jms-destination";
@@ -403,6 +449,8 @@ public interface CommonAttributes {
     String SHARED_STORE_MASTER = "shared-store-master";
     String SHARED_STORE_SLAVE = "shared-store-slave";
     String SLAVE = "slave";
+    String SOCKET_BROADCAST_GROUP = "socket-broadcast-group";
+    String SOCKET_DISCOVERY_GROUP = "socket-discovery-group";
     String SOURCE = "source";
     String STARTED = "started";
     String STATIC_CONNECTORS = "static-connectors";
@@ -417,4 +465,12 @@ public interface CommonAttributes {
     String VERSION = "version";
     String XA = "xa";
     String XA_TX = "XATransaction";
+
+    static void renameChannelToCluster(ModelNode operation) {
+        // Handle jgroups-channel -> jgroups-cluster rename
+        if (!operation.hasDefined(CommonAttributes.JGROUPS_CLUSTER.getName()) && operation.hasDefined(CommonAttributes.JGROUPS_CHANNEL.getName())) {
+            operation.get(CommonAttributes.JGROUPS_CLUSTER.getName()).set(operation.get(CommonAttributes.JGROUPS_CHANNEL.getName()));
+            operation.remove(CommonAttributes.JGROUPS_CHANNEL.getName());
+        }
+    }
 }
