@@ -35,6 +35,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Supplier;
 
 import javax.security.auth.Subject;
 import javax.security.jacc.PolicyContext;
@@ -60,6 +61,7 @@ import org.jboss.security.audit.AuditLevel;
 import org.jboss.security.audit.AuditManager;
 import org.jboss.security.authorization.resources.EJBResource;
 import org.jboss.security.identity.Identity;
+import org.jboss.security.identity.RoleGroup;
 import org.jboss.security.identity.plugins.SimpleIdentity;
 import org.jboss.security.identity.plugins.SimpleRoleGroup;
 import org.jboss.security.javaee.AbstractEJBAuthorizationHelper;
@@ -69,13 +71,22 @@ import org.wildfly.security.auth.server.IdentityCredentials;
 import org.wildfly.security.auth.server.SecurityDomain;
 import org.wildfly.security.auth.server.SecurityIdentity;
 import org.wildfly.security.credential.PasswordCredential;
+import org.wildfly.security.manager.WildFlySecurityManager;
 import org.wildfly.security.password.interfaces.ClearPassword;
 
 /**
+ * This {@code SimpleSecurityManager} also implements {@code Supplier<ServerSecurityManager>} so that
+ * the corresponding {@code get()} method can return a new instance which delegates to {@code this}.
+ *
  * @author <a href="mailto:cdewolf@redhat.com">Carlo de Wolf</a>
  * @author <a href="mailto:darran.lofthouse@jboss.com">Darran Lofthouse</a>
  */
-public class SimpleSecurityManager implements ServerSecurityManager {
+public class SimpleSecurityManager implements ServerSecurityManager, Supplier<ServerSecurityManager> {
+    protected static final PrivilegedAction<SecurityContext> GET_SECURITY_CONTEXT = new PrivilegedAction<SecurityContext>() {
+        public SecurityContext run() {
+            return SecurityContextAssociation.getSecurityContext();
+        }
+    };
     private ThreadLocalStack<SecurityContext> contexts = new ThreadLocalStack<SecurityContext>();
     /**
      * Indicates if we propagate previous SecurityContext informations to the current one or not.
@@ -88,21 +99,19 @@ public class SimpleSecurityManager implements ServerSecurityManager {
     public SimpleSecurityManager() {
     }
 
-    public SimpleSecurityManager(SimpleSecurityManager delegate) {
+    SimpleSecurityManager(SimpleSecurityManager delegate) {
         this.securityManagement = delegate.securityManagement;
         this.propagate = false;
     }
 
-    private ISecurityManagement securityManagement = null;
-
-    private PrivilegedAction<SecurityContext> securityContext() {
-        return new PrivilegedAction<SecurityContext>() {
-            public SecurityContext run() {
-                return SecurityContextAssociation.getSecurityContext();
-            }
-        };
+    public ServerSecurityManager get() {
+        return new SimpleSecurityManager(this);
     }
 
+
+
+    private ISecurityManagement securityManagement = null;
+
     private SecurityContext establishSecurityContext(final String securityDomain) {
         // Do not use SecurityFactory.establishSecurityContext, its static init is broken.
         try {
@@ -122,7 +131,12 @@ public class SimpleSecurityManager implements ServerSecurityManager {
     }
 
     public Principal getCallerPrincipal() {
-        final SecurityContext securityContext = doPrivileged(securityContext());
+        final SecurityContext securityContext;
+        if (WildFlySecurityManager.isChecking()) {
+            securityContext = doPrivileged(GET_SECURITY_CONTEXT);
+        } else {
+            securityContext = SecurityContextAssociation.getSecurityContext();
+        }
         if (securityContext == null) {
             return getUnauthenticatedIdentity().asPrincipal();
         }
@@ -138,7 +152,12 @@ public class SimpleSecurityManager implements ServerSecurityManager {
     }
 
     public Subject getSubject() {
-        final SecurityContext securityContext = doPrivileged(securityContext());
+        final SecurityContext securityContext;
+        if (WildFlySecurityManager.isChecking()) {
+            securityContext = doPrivileged(GET_SECURITY_CONTEXT);
+        } else {
+            securityContext = SecurityContextAssociation.getSecurityContext();
+        }
         if (securityContext != null) {
             return getSubjectInfo(securityContext).getAuthenticatedSubject();
         }
@@ -192,8 +211,12 @@ public class SimpleSecurityManager implements ServerSecurityManager {
      */
     public boolean isCallerInRole(final String ejbName, final String policyContextID, final Object incommingMappedRoles,
                                   final Map<String, Collection<String>> roleLinks, final String... roleNames) {
-
-        final SecurityContext securityContext = doPrivileged(securityContext());
+        final SecurityContext securityContext;
+        if (WildFlySecurityManager.isChecking()) {
+            securityContext = doPrivileged(GET_SECURITY_CONTEXT);
+        } else {
+            securityContext = SecurityContextAssociation.getSecurityContext();
+        }
         if (securityContext == null) {
             return false;
         }
@@ -245,8 +268,12 @@ public class SimpleSecurityManager implements ServerSecurityManager {
     }
 
     public boolean authorize(String ejbName, CodeSource ejbCodeSource, String ejbMethodIntf, Method ejbMethod, Set<Principal> methodRoles, String contextID) {
-
-        final SecurityContext securityContext = doPrivileged(securityContext());
+        final SecurityContext securityContext;
+        if (WildFlySecurityManager.isChecking()) {
+            securityContext = doPrivileged(GET_SECURITY_CONTEXT);
+        } else {
+            securityContext = SecurityContextAssociation.getSecurityContext();
+        }
         if (securityContext == null) {
             return false;
         }
@@ -282,8 +309,7 @@ public class SimpleSecurityManager implements ServerSecurityManager {
         contexts.push(previous);
         SecurityContext current = establishSecurityContext(securityDomain);
         if (propagate && previous != null) {
-            current.setSubjectInfo(getSubjectInfo(previous));
-
+            propagateSubject(current,previous);
             // If the outgoing run-as identity is not set, take the incoming
             // run-as identity and propagate it (it could be null which is fine)
             if( previous.getOutgoingRunAs() != null ) {
@@ -336,7 +362,7 @@ public class SimpleSecurityManager implements ServerSecurityManager {
         contexts.push(previous);
         SecurityContext current = establishSecurityContext(securityDomain);
         if (propagate &&  previous != null) {
-            current.setSubjectInfo(getSubjectInfo(previous));
+            propagateSubject(current,previous);
             current.setIncomingRunAs(previous.getOutgoingRunAs());
         }
 
@@ -349,6 +375,25 @@ public class SimpleSecurityManager implements ServerSecurityManager {
         }
     }
 
+    private void propagateSubject(final SecurityContext target, final SecurityContext source) {
+        final SecurityContextUtil previousUtil = source.getUtil();
+        final SecurityContextUtil currentUtil = target.getUtil();
+        if(target.getSecurityDomain() != null && source.getSecurityDomain() != null && target.getSecurityDomain().equals(source.getSecurityDomain())){
+            target.setSubjectInfo(source.getSubjectInfo());
+        } else {
+            currentUtil.createSubjectInfo(previousUtil.getUserPrincipal(), previousUtil.getCredential(), previousUtil.getSubject());
+            if (previousUtil.getRoles() != null) {
+                try {
+                    currentUtil.setRoles((RoleGroup) previousUtil.getRoles().clone());
+                } catch (CloneNotSupportedException e) {
+                    // should not happen, RoleGroup supports clone.
+                    throw new RuntimeException(e);
+                }
+            }
+            currentUtil.setIdentities(previousUtil.getIdentities(Identity.class));
+        }
+    }
+
     public void authenticate() {
         authenticate(null, null, null);
     }
@@ -359,7 +404,8 @@ public class SimpleSecurityManager implements ServerSecurityManager {
 
         // skip reauthentication if the current context already has an authenticated subject (copied from the previous context
         // upon creation - see push method) and both contexts use the same security domain or there is an incoming RunAs of RunAsIdentity type
-        boolean skipReauthentication = current.getSubjectInfo() != null && current.getSubjectInfo().getAuthenticatedSubject() != null && (
+        boolean skipReauthentication = current.getSubjectInfo() != null && current.getSubjectInfo().getAuthenticatedSubject() != null &&
+                !current.getSubjectInfo().getAuthenticatedSubject().getPrincipals().isEmpty() && (
                         (previous != null && current.getSecurityDomain().equals(previous.getSecurityDomain())) ||
                         current.getIncomingRunAs() instanceof RunAsIdentity
                 );
